"""Provides with a memento for UnitOfWork.
"""

__all__ = ["UnitOfWorkMemento"]


class UnitOfWorkMemento:
    """Base class for UnitOfWorkMemento.

    @DynamicAttrs"""

    def __init__(self, **kwargs):
        pass
