"""Provides with Entity class.
"""


class Entity:
    """Entity class.
    Used as base class when making dataclasses from declarative bases.

    @DynamicAttrs"""

    def __init__(self, *args, **kwargs):
        pass
